@@ -1,9 +1,11 @@
 ;;; ghub-graphql.el --- access Github API using GrapthQL  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2016-2020  Jonas Bernoulli
+;; Copyright (C) 2016-2021  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Homepage: https://github.com/magit/ghub
+;; Keywords: tools
+;; SPDX-License-Identifier: GPL-3.0-or-later
 
 ;; This file is not part of GNU Emacs.
 
@@ -69,7 +71,7 @@ behave as for `ghub-request' (which see)."
 
 ;;; Api (drafts)
 
-(defconst ghub-fetch-repository
+(defconst ghub-fetch-repository-sparse
   '(query
     (repository
      [(owner $owner String!)
@@ -92,7 +94,12 @@ behave as for `ghub-request' (which see)."
      hasWikiEnabled
      (licenseInfo name)
      (stargazers totalCount)
-     (watchers totalCount)
+     (watchers totalCount))))
+
+(defconst ghub-fetch-repository
+  `(query
+    (repository
+     ,@(cdr (cadr ghub-fetch-repository-sparse))
      (assignableUsers [(:edges t)]
                       id
                       login
@@ -115,8 +122,8 @@ behave as for `ghub-request' (which see)."
                      (comments  [(:edges t)]
                                 databaseId
                                 (author login)
-	                        createdAt
-	                        updatedAt
+                                createdAt
+                                updatedAt
                                 body)
                      (labels    [(:edges t)]
                                 id))
@@ -154,9 +161,11 @@ behave as for `ghub-request' (which see)."
                      body
                      (baseRef name
                               (repository nameWithOwner))
+                     baseRefOid
                      (headRef name
                               (repository (owner login)
                                           nameWithOwner))
+                     headRefOid
                      (assignees [(:edges t)]
                                 id)
                      (reviewRequests [(:edges t)]
@@ -165,18 +174,49 @@ behave as for `ghub-request' (which see)."
                                 databaseId
                                 (author login)
                                 createdAt
-	                        updatedAt
-	                        body)
+                                updatedAt
+                                body)
                      (labels    [(:edges t)]
                                 id)))))
 
+(defconst ghub-fetch-repository-review-threads
+  '(query
+    (repository
+     [(owner $owner String!)
+      (name  $name  String!)]
+     (pullRequests   [(:edges t)
+                      (:singular pullRequest number)
+                      (orderBy ((field UPDATED_AT) (direction DESC)))]
+                     number
+                     baseRefOid
+                     headRefOid
+                     (reviewThreads [(:edges t)]
+                                    id
+                                    line
+                                    originalLine
+                                    diffSide
+                                    (resolvedBy login)
+                                    (comments [(:edges t)]
+                                              id
+                                              databaseId
+                                              (author login)
+                                              createdAt
+                                              updatedAt
+                                              body
+                                              (replyTo databaseId)
+                                              (originalCommit oid)
+                                              path))))))
+
 (cl-defun ghub-fetch-repository (owner name callback
                                        &optional until
-                                       &key username auth host forge errorback)
+                                       &key username auth host forge
+                                       headers errorback sparse)
   "Asynchronously fetch forge data about the specified repository.
 Once all data has been collected, CALLBACK is called with the
 data as the only argument."
-  (ghub--graphql-vacuum ghub-fetch-repository
+  (ghub--graphql-vacuum (if sparse
+                            ghub-fetch-repository-sparse
+                          ghub-fetch-repository)
                         `((owner . ,owner)
                           (name  . ,name))
                         callback until
@@ -185,11 +225,13 @@ data as the only argument."
                         :auth     auth
                         :host     host
                         :forge    forge
+                        :headers  headers
                         :errorback errorback))
 
 (cl-defun ghub-fetch-issue (owner name number callback
                                   &optional until
-                                  &key username auth host forge errorback)
+                                  &key username auth host forge
+                                  headers errorback)
   "Asynchronously fetch forge data about the specified issue.
 Once all data has been collected, CALLBACK is called with the
 data as the only argument."
@@ -204,11 +246,13 @@ data as the only argument."
                         :auth     auth
                         :host     host
                         :forge    forge
+                        :headers  headers
                         :errorback errorback))
 
 (cl-defun ghub-fetch-pullreq (owner name number callback
                                     &optional until
-                                    &key username auth host forge errorback)
+                                    &key username auth host forge
+                                    headers errorback)
   "Asynchronously fetch forge data about the specified pull-request.
 Once all data has been collected, CALLBACK is called with the
 data as the only argument."
@@ -223,6 +267,28 @@ data as the only argument."
                         :auth     auth
                         :host     host
                         :forge    forge
+                        :headers  headers
+                        :errorback errorback))
+
+(cl-defun ghub-fetch-review-threads (owner name number callback
+                                           &optional until
+                                           &key username auth host forge
+                                           headers errorback)
+  "Asynchronously fetch forge data about the review threads from a pull-request.
+Once all data has been collected, CALLBACK is called with the
+data as the only argument."
+  (ghub--graphql-vacuum (ghub--graphql-prepare-query
+                         ghub-fetch-repository-review-threads
+                         `(repository pullRequests (pullRequest . ,number)))
+                        `((owner . ,owner)
+                          (name  . ,name))
+                        callback until
+                        :narrow   '(repository pullRequest)
+                        :username username
+                        :auth     auth
+                        :host     host
+                        :forge    forge
+                        :headers  headers
                         :errorback errorback))
 
 ;;; Internal
@@ -241,7 +307,7 @@ data as the only argument."
 (cl-defun ghub--graphql-vacuum (query variables callback
                                       &optional until
                                       &key narrow username auth host forge
-                                      errorback)
+                                      headers errorback)
   "Make a GraphQL request using QUERY and VARIABLES.
 See Info node `(ghub)GraphQL Support'."
   (unless host
@@ -256,7 +322,7 @@ See Info node `(ghub)GraphQL Support'."
                             (substring host 0 -3)
                           host)))
     :method    "POST"
-    :headers   (ghub--headers nil host auth username forge)
+    :headers   (ghub--headers headers host auth username forge)
     :handler   'ghub--graphql-handle-response
     :query     query
     :variables variables
@@ -316,10 +382,8 @@ See Info node `(ghub)GraphQL Support'."
               (dolist (elt alist)
                 (cond ((keywordp (car elt)))
                       ((= (length elt) 3)
-                       (push (list (nth 0 elt)
-                                   (nth 1 elt)) vars)
-                       (push (list (nth 1 elt)
-                                   (nth 2 elt)) variables))
+                       (push (list (nth 0 elt) (nth 1 elt)) vars)
+                       (push (list (nth 1 elt) (nth 2 elt)) variables))
                       ((= (length elt) 2)
                        (push elt vars))))
               (setq loc (treepy-replace loc (vconcat (nreverse vars)))))))
@@ -418,7 +482,7 @@ See Info node `(ghub)GraphQL Support'."
         `(,(car single)
           ,(vector (list (cadr single) (cdr (car lineage))))
           ,@(if (cdr lineage)
-               (ghub--graphql-narrow-query child (cdr lineage) cursor)
+                (ghub--graphql-narrow-query child (cdr lineage) cursor)
               child)))
     (let* ((child  (or (assq (car lineage) (cdr query))
                        ;; Alias
