@@ -1,10 +1,11 @@
 ;;; ghub.el --- minuscule client libraries for Git forge APIs  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2016-2020  Jonas Bernoulli
+;; Copyright (C) 2016-2021  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Homepage: https://github.com/magit/ghub
 ;; Keywords: tools
+;; SPDX-License-Identifier: GPL-3.0-or-later
 
 ;; This file is not part of GNU Emacs.
 
@@ -411,12 +412,12 @@ this function is called with nil for PAYLOAD."
               (setq-default ghub-response-headers headers))
             page)
         (cdr (assq 'link-alist ghub-response-headers)))
-  (when-let ((rels (cdr (assoc "Link" (or headers ghub-response-headers)))))
-    (mapcar (lambda (elt)
-              (pcase-let ((`(,url ,rel) (split-string elt "; ")))
-                (cons (intern (substring rel 5 -1))
-                      (substring url 1 -1))))
-            (split-string rels ", ")))))
+    (when-let ((rels (cdr (assoc "Link" (or headers ghub-response-headers)))))
+      (mapcar (lambda (elt)
+                (pcase-let ((`(,url ,rel) (split-string elt "; ")))
+                  (cons (intern (substring rel 5 -1))
+                        (substring url 1 -1))))
+              (split-string rels ", ")))))
 
 (cl-defun ghub-repository-id (owner name &key username auth host forge noerror)
   "Return the id of the specified repository.
@@ -470,8 +471,8 @@ kludge.
 - For any other non-nil value use the kludge, if and only if we
   believe that doing so is the correct thing to do.
 
-The default value of this variable is either nil or `forge'.  It
-is `forge' if using libgnutls >=3.6.3 (the version introducing
+The default value of this variable is either nil or `force'.  It
+is `force' if using libgnutls >=3.6.3 (the version introducing
 TLS1.3); AND also using Emacs < 26.3 and/or macOS (any version).
 
 If the value is any other non-nil value, then `ghub--retrieve'
@@ -651,10 +652,10 @@ and https://debbugs.gnu.org/cgi/bugreport.cgi?bug=34341.")
   (and payload
        (progn
          (unless (stringp payload)
-           ;; Unfortunately `json-encode-list' may modify the input.
+           ;; Unfortunately `json-encode' may modify the input.
            ;; See https://debbugs.gnu.org/cgi/bugreport.cgi?bug=40693.
            ;; and https://github.com/magit/forge/issues/267
-           (setq payload (json-encode-list (copy-tree payload))))
+           (setq payload (json-encode (copy-tree payload))))
          (encode-coding-string payload 'utf-8))))
 
 (defun ghub--url-encode-params (params)
@@ -815,13 +816,13 @@ or (info \"(ghub)Getting Started\") for instructions.
              (user-error "The empty string is not a valid username")
            (call-process
             "git" nil nil nil "config"
-            (and (eq (read-char-choice
-                      (format
-                       "Set %s=%s [g]lobally (recommended) or [l]ocally? "
-                       var user)
-                      (list ?g ?l))
-                     ?g)
-                 "--global")
+            (if (eq (read-char-choice
+                     (format "Set %s=%s [g]lobally (recommended) or [l]ocally? "
+                             var user)
+                     (list ?g ?l))
+                    ?g)
+                "--global"
+              "--local")
             var user)
            user))))))
 
@@ -847,18 +848,21 @@ won't see the secret from a line that is followed by a commented
 line."
     (save-match-data (funcall fn))))
 
-(advice-add 'url-http-handle-authentication :around
-            'url-http-handle-authentication@unauthorized-bugfix)
-(defun url-http-handle-authentication@unauthorized-bugfix (fn proxy)
-  "If authorization failed then don't try again but fail properly.
+(progn ; (when (< emacs-major-version 28)
+  ;; Fixed by Emacs commit 0b98ea5fbe276c67206896dca111c000f984ee0f,
+  ;; but keep this advice until 28.1 is released.
+  (advice-add 'url-http-handle-authentication :around
+              'url-http-handle-authentication@unauthorized-bugfix)
+  (defun url-http-handle-authentication@unauthorized-bugfix (fn proxy)
+    "If authorization failed then don't try again but fail properly.
 For Emacs 27.1 prevent a useful `http' error from being replaced
 by a generic one that omits all useful information.  For earlier
 releases prevent a new request from being made, which would
 either result in an infinite loop or (e.g. in the case of `ghub')
 the user being asked for their name."
-  (if (assoc "Authorization" url-http-extra-headers)
-      t ; Return "success", here also known as "successfully failed".
-    (funcall fn proxy)))
+    (if (assoc "Authorization" url-http-extra-headers)
+        t ; Return "success", here also known as "successfully failed".
+      (funcall fn proxy))))
 
 ;;; _
 (provide 'ghub)
